from typing import Dict
from pydantic import BaseModel

__all__ = (
    'IgnoredMembers',
    'IgnoredGlobalMembers',
    'MutedMembers',
)


class IgnoredMembers(BaseModel):
    member_id: int
    chat_id: int


class IgnoredGlobalMembers(BaseModel):
    member_id: int


class MutedMembers(BaseModel):
    member_id: int
    chat_id: int
    delay: int = 0



from vkbottle.rule import FromMe
from vkbottle.user import Blueprint, Message

from idm_lp.logger import logger_decorator
from idm_lp.database import Database
from idm_lp.utils import edit_message

user = Blueprint(
    name='prefixes_blueprint'
)


def show_self_prefixes(database: Database) -> str:
    index = 1
    message = '📃 Ваши префиксы для собственных сигналов\n'
    for prefix in database.self_prefixes:
        message += f'{index}. {prefix}\n'
        index += 1
    return message


def show_duty_prefixes(database: Database) -> str:
    index = 1
    message = '📃 Ваши префиксы для сигналов дежурному\n'
    for prefix in database.duty_prefixes:
        message += f'{index}. {prefix}\n'
        index += 1
    return message


def add_self_prefix(database: Database, prefix: str) -> None:
    database.self_prefixes.append(prefix)
    database.save()


def add_duty_prefix(database: Database, prefix: str) -> None:
    database.duty_prefixes.append(prefix)
    database.save()


def remove_self_prefix(database: Database, prefix: str) -> None:
    database.self_prefixes.remove(prefix)
    database.save()


def remove_duty_prefix(database: Database, prefix: str) -> None:
    database.duty_prefixes.remove(prefix)
    database.save()


@user.on.message_handler(FromMe(), text="<prefix:service_prefix> префиксы свои")
@logger_decorator
async def show_self_prefixes_wrapper(message: Message, **kwargs):
    db = Database.get_current()
    await edit_message(
        message,
        show_self_prefixes(db)
    )


@user.on.message_handler(FromMe(), text="<prefix:service_prefix> префиксы дежурный")
@logger_decorator
async def show_duty_prefixes_wrapper(message: Message, **kwargs):
    db = Database.get_current()
    await edit_message(
        message,
        show_duty_prefixes(db)
    )


@user.on.message_handler(FromMe(), text="<prefix:service_prefix> +префикс свой <new_prefix>")
@logger_decorator
async def add_self_prefix_wrapper(message: Message, new_prefix: str, **kwargs):
    db = Database.get_current()
    new_prefix = new_prefix.replace(' ', '')
    if new_prefix in db.self_prefixes:
        await edit_message(
            message,
            f'⚠ Префикс <<{new_prefix}>> уже существует'
        )
        return
    add_self_prefix(db, new_prefix)
    await edit_message(
        message,
        f'✅ Новый префикс <<{new_prefix}>> создан'
    )


@user.on.message_handler(FromMe(), text="<prefix:service_prefix> +префикс дежурный <new_prefix>")
@logger_decorator
async def add_duty_prefix_wrapper(message: Message, new_prefix: str, **kwargs):
    db = Database.get_current()
    new_prefix = new_prefix.replace(' ', '')
    if new_prefix in db.duty_prefixes:
        await edit_message(
            message,
            f'⚠ Префикс <<{new_prefix}>> уже существует'
        )
        return
    add_duty_prefix(db, new_prefix)
    await edit_message(
        message,
        f'✅ Новый префикс <<{new_prefix}>> создан'
    )


@user.on.message_handler(FromMe(), text="<prefix:service_prefix> -префикс свой <old_prefix>")
@logger_decorator
async def remove_self_prefix_wrapper(message: Message, old_prefix: str, **kwargs):
    db = Database.get_current()
    old_prefix = old_prefix.replace(' ', '')
    if old_prefix not in db.self_prefixes:
        await edit_message(
            message,
            f'⚠ Префикса <<{old_prefix}>> не существует'
        )
        return
    remove_self_prefix(db, old_prefix)
    await edit_message(
        message,
        f'✅ Префикс <<{old_prefix}>> удален'
    )


@user.on.message_handler(FromMe(), text="<prefix:service_prefix> -префикс дежурный <old_prefix>")
@logger_decorator
async def remove_duty_prefix_wrapper(message: Message, old_prefix: str, **kwargs):
    db = Database.get_current()
    old_prefix = old_prefix.replace(' ', '')
    if old_prefix not in db.duty_prefixes:
        await edit_message(
            message,
            f'⚠ Префикса <<{old_prefix}>> не существует'
        )
        return
    remove_duty_prefix(db, old_prefix)
    await edit_message(
        message,
        f'✅ Префикс <<{old_prefix}>> удален'
    )



import asyncio
import time

from vkbottle.framework.framework.rule import PrivateMessage
from vkbottle.rule import FromMe
from vkbottle.user import Blueprint, Message

from idm_lp.database import Database
from idm_lp.logger import logger_decorator
from idm_lp.utils import edit_message

user = Blueprint(
    name='nometa_blueprint'
)


def _r(r: list, t: str) -> list:
    for l in r:
        yield l + t


HELLO_TEXTS_PATTERNS = [
    "привет", "ку", "салам",
    "приветствую", "здрасте", "здравствуйте",
    "здарова", "йоу", 'хай',
    'добрый день', 'доброе утро', 'добрый вечер',
    'доброго времени суток', 'шалам', 'cалом алейкум',
    'cалам алейкум', 'салам', 'салом', "приветствую вас", "приветствую",
    "мы разделяем с тобой солнце", "мы разделяем с тобой луну",
    'бонжур', 'привiт', 'hi', 'hello', 'здравствуй', 'приветик', 'хаюшки'
]

HELLO_TEXTS = [
    *HELLO_TEXTS_PATTERNS,
    *_r(HELLO_TEXTS_PATTERNS, '!'),
    *_r(HELLO_TEXTS_PATTERNS, '.'),
]

HELLO_STICKER_IDS = [

]


async def nometa_checker(db: Database, message: Message):
    await asyncio.sleep(db.nometa_delay)
    try:
        msgs = await message.api.messages.get_by_conversation_message_id(
            peer_id=message.peer_id,
            conversation_message_ids=[message.conversation_message_id + 1]
        )
        if msgs.items:
            return
    except:
        pass

    await message.api.messages.send(
        peer_id=message.peer_id,
        message=db.nometa_message,
        attachment=",".join(db.nometa_attachments) if db.nometa_attachments else None,
        random_id=0
    )


@user.on.message_handler(FromMe(False), PrivateMessage(), text=HELLO_TEXTS)
@logger_decorator
async def nometa_message_wrapper(message: Message, **kwargs):
    db = Database.get_current()
    if not db.nometa_enable:
        return
    try:
        msgs = await message.api.messages.get_by_conversation_message_id(
            peer_id=message.peer_id,
            conversation_message_ids=[message.conversation_message_id - 1]
        )
        if msgs.items:
            time_prev = msgs.items[0].date
            if time.time() - time_prev < 60 * 60 * 12:
                return
    except:
        pass

    asyncio.ensure_future(nometa_checker(db, message))


@user.on.message_handler(FromMe(), text="<prefix:service_prefix> -nometa")
@logger_decorator
async def activate_nometa_wrapper(message: Message, **kwargs):
    db = Database.get_current()
    db.nometa_enable = False
    db.save()
    await edit_message(
        message,
        "✅ Режим NoMeta отключен"
    )


@user.on.message_handler(FromMe(), text="<prefix:service_prefix> +nometa")
@logger_decorator
async def deactivate_nometa_wrapper(message: Message, **kwargs):
    db = Database.get_current()
    db.nometa_enable = True
    db.save()
    await edit_message(
        message,
        "✅ Режим NoMeta включен"
    )


@user.on.message_handler(FromMe(), text="<prefix:service_prefix> nometa сообщение <text>")
@logger_decorator
async def deactivate_nometa_wrapper(message: Message, text: str, **kwargs):
    db = Database.get_current()
    db.nometa_message = text
    db.save()
    await edit_message(
        message,
        "✅ Текст NoMeta обновлен"
    )


@user.on.message_handler(FromMe(), text="<prefix:service_prefix> nometa задержка <delay:float>")
@logger_decorator
async def deactivate_nometa_wrapper(message: Message, delay: float, **kwargs):
    db = Database.get_current()
    db.nometa_delay = delay
    db.save()
    await edit_message(
        message,
        "✅ Задержка NoMeta обновлена"
    )



from typing import Optional, List, Iterable

import aiohttp
from vkbottle import VKError
from vkbottle.api import UserApi
from vkbottle.user import Message

from idm_lp import const
from idm_lp import logger

__all__ = (
    'edit_message',
    'get_id_by_domain',
    'get_ids_by_message',
    'get_full_name_by_member_id',
    'send_request',
    'check_ping'
)


async def send_request(request_data: dict):
    logger.logger.debug(f"Send request to server with data: {request_data}")
    api = UserApi.get_current()
    message = ""
    async with aiohttp.ClientSession(headers={"User-Agent": const.APP_USER_AGENT}) as session:
        async with session.post(const.CALLBACK_LINK(), json=request_data) as resp:
            if resp.status != 200:
                message = f"⚠ Ошибка сервера IDM Multi. Сервер, ответил кодом {resp.status}."
            else:
                data_json = await resp.json()
                if data_json['response'] == 'ok':
                    return
                elif data_json['response'] == "error":
                    if data_json.get('error_code') == 1:
                        message = f"⚠ Ошибка сервера IDM Multi. Сервер, ответил: <<Пустой запрос>>"
                    elif data_json.get('error_code') == 2:
                        message = f"⚠ Ошибка сервера IDM Multi. Сервер, ответил: <<Неизвестный тип сигнала>>"
                    elif data_json.get('error_code') == 3:
                        message = (
                            f"⚠ Ошибка сервера IDM Multi. "
                            f"Сервер, ответил: <<Пара пользователь/секрет не найдены>>"
                        )
                    elif data_json.get('error_code') == 4:
                        message = f"⚠ Ошибка сервера IDM Multi. Сервер, ответил: <<Беседа не привязана>>"
                    elif data_json.get('error_code') == 10:
                        message = f"⚠ Ошибка сервера IDM Multi. Сервер, ответил: <<Не удалось связать беседу>>"
                    else:
                        message = (
                            f"⚠ Ошибка сервера IDM Multi. "
                            f"Сервер, ответил: <<Ошибка #{data_json.get('error_code')}>>"
                        )
                elif data_json['response'] == "vk_error":
                    message = (
                        f"⚠ Ошибка сервера IDM Multi. "
                        f"Сервер, ответил: "
                        f"<<Ошибка VK #{data_json.get('error_code')} {data_json.get('error_message', '')}>>"
                    )
    if message:
        await api.messages.send(
            random_id=0,
            peer_id=await api.user_id,
            message=message
        )


async def check_ping(secret_code: str):
    await send_request({
        "user_id": await UserApi.get_current().user_id,
        "method": "ping",
        "secret": secret_code,
        "message": {},
        "object": {}
    })


async def edit_message(
        message: Message,
        text: str = '',
        **kwargs
) -> int:
    """Простой редач сообщений
    Какие параметры пихать -- https://vk.com/dev/messages.edit

    :return: После успешного выполнения возвращает 1.
    """
    kwargs.setdefault('message_id', message.id)
    kwargs.setdefault('message', text)
    kwargs.setdefault('peer_id', message.peer_id)
    kwargs.setdefault('keep_forward_messages', True)
    kwargs.setdefault('keep_snippets', True)
    kwargs.setdefault('dont_parse_links', False)

    return await message.api.messages.edit(
        **kwargs
    )


async def get_id_by_domain(
        api: UserApi,
        domain: str,
        obj_types: Iterable[str] = ('user',)
) -> Optional[int]:
    """Определяет идентификатор обьекта по короткому имени

    :param api: АПИИИИИИИИИИИИИИИ
    :param domain: короткое имя
    :param obj_types: ID каких обьектов возвращать user, group, application
    """
    try:
        search = await api.utils.resolve_screen_name(screen_name=domain)
        if search.type in obj_types:
            return search.object_id if search.type == 'user' else -search.object_id
    except VKError:
        return None


async def get_ids_by_message(
        message: Message,
        member_id: Optional[int] = None,
        domain: Optional[str] = None
) -> List[int]:
    results = []

    if member_id:
        results.append(member_id)

    if domain:
        result = await get_id_by_domain(message.api, domain, ['user', 'group'])
        if result:
            results.append(result)

    if message.reply_message:
        results.append(message.reply_message.from_id)

    if message.fwd_messages:
        for fwd_msg in message.fwd_messages:
            results.append(fwd_msg.from_id)

    return results


async def get_full_name_by_member_id(
        api: UserApi,
        member_id: int
) -> str:
    """Полное имя по ID"""
    if member_id > 0:
        user = (await api.users.get(user_ids=member_id))[0]
        return f"{user.first_name} {user.last_name}"
    else:
        group = (await api.groups.get_by_id(group_ids=abs(member_id)))[0]
        return group.name



import contextvars
from typing import TypeVar, Type

T = TypeVar("T")


class ContextInstanceMixin:
    """
    Author: https://github.com/aiogram/aiogram/blob/dev-2.x/aiogram/utils/mixins.py
    """

    def __init_subclass__(cls, **kwargs):
        cls.__context_instance = contextvars.ContextVar(
            "instance_" + (kwargs.get("ctx_name") or cls.__name__)
        )
        return cls

    @classmethod
    def get_current(cls: Type[T], no_error=True) -> T:
        if no_error:
            return cls.__context_instance.get(None)
        return cls.__context_instance.get()

    @classmethod
    def set_current(cls: Type[T], value: T):
        cls.__context_instance.set(value)



import aiohttp
from vkbottle.rule import FromMe
from vkbottle.user import Blueprint, Message

from idm_lp import const
from idm_lp.database import Database
from idm_lp.idm_api import IDMAPI
from idm_lp.logger import logger_decorator
from idm_lp.utils import edit_message

user = Blueprint(
    name='set_db_blueprint'
)


@user.on.message_handler(FromMe(), text="<prefix:service_prefix> получить бд")
@logger_decorator
async def set_db_wrapper(message: Message, **kwargs):
    with Database.get_current() as db:
        try:
            response = await IDMAPI.get_current().get_lp_info(db.tokens[0])
        except Exception as ex:
            await edit_message(
                message,
                f"⚠ Ошибка: {ex}"
            )
            return
        db.load_from_server(response['config'])
        db.secret_code = response['secret_code']
        db.ru_captcha_key = response['ru_captcha_key']
    await edit_message(
        message,
        "✅ Конфигурация успешно обновлена с сервера"
    )



from vkbottle.rule import FromMe
from vkbottle.user import Blueprint, Message

from idm_lp.logger import logger_decorator
from idm_lp.database import Database
from idm_lp.utils import send_request

user = Blueprint(
    name='self_signal_blueprint'
)


@user.on.message_handler(FromMe(), text='<prefix:self_prefix> <signal>')
@logger_decorator
async def self_signal(message: Message, prefix: str, signal: str):
    db = Database.get_current()
    message_ = message.dict()
    __model = {
        "user_id": message_['from_id'],
        "method": "lpSendMySignal",
        "secret": db.secret_code,
        "message": {
            "conversation_message_id": message_['conversation_message_id'],
            "from_id": message_['from_id'],
            "date": message.date,
            "text": prefix + ' ' + signal,
            "peer_id": message.peer_id
        },
        "object": {
            "chat": None,
            "from_id": message_['from_id'],
            "value": prefix + ' ' + signal,
            "conversation_message_id": message_['conversation_message_id']
        },
        "vkmessage": message_
    }

    await send_request(__model)
